@@ -65,7 +65,7 @@
 ;;; (def pprint-simple-list (formatter-out "~:<~@{~w~^ ~_~}~:>"))
 (defn- pprint-simple-list [alis]
   (pprint-logical-block :prefix "(" :suffix ")"
-    (loop [alis (seq alis)]
+    (print-length-loop [alis (seq alis)]
       (when alis
 	(write-out (first alis))
 	(when (next alis)
@@ -80,7 +80,7 @@
 ;;; (def pprint-vector (formatter-out "~<[~;~@{~w~^ ~_~}~;]~:>"))
 (defn- pprint-vector [avec]
   (pprint-logical-block :prefix "[" :suffix "]"
-    (loop [aseq (seq avec)]
+    (print-length-loop [aseq (seq avec)]
       (when aseq
 	(write-out (first aseq))
 	(when (next aseq)
@@ -93,12 +93,13 @@
 ;;; (def pprint-map (formatter-out "~<{~;~@{~<~w~^ ~_~w~:>~^, ~_~}~;}~:>"))
 (defn- pprint-map [amap]
   (pprint-logical-block :prefix "{" :suffix "}"
-    (loop [aseq (seq amap)]
+    (print-length-loop [aseq (seq amap)]
       (when aseq
 	(pprint-logical-block 
           (write-out (ffirst aseq))
           (.write ^java.io.Writer *out* " ")
           (pprint-newline :linear)
+          (set! *current-length* 0)     ; always print both parts of the [k v] pair
           (write-out (fnext (first aseq))))
         (when (next aseq)
           (.write ^java.io.Writer *out* ", ")
@@ -107,8 +108,6 @@
 
 (def ^{:private true} pprint-set (formatter-out "~<#{~;~@{~w~^ ~:_~}~;}~:>"))
 
-;;; TODO: don't block on promise (currently impossible)
-
 (def ^{:private true} 
      type-map {"core$future_call" "Future",
                "core$promise" "Promise"})
@@ -133,6 +132,7 @@
                            (pprint-newline :linear)
                            (write-out (cond 
                                        (and (future? o) (not (future-done? o))) :pending
+                                       (and (instance? clojure.lang.IPending o) (not (.isRealized o))) :not-delivered
                                        :else @o)))))
 
 (def ^{:private true} pprint-pqueue (formatter-out "~<<-(~;~@{~w~^ ~_~}~;)-<~:>"))
@@ -219,7 +219,7 @@
 
 (defn- pprint-binding-form [binding-vec]
   (pprint-logical-block :prefix "[" :suffix "]"
-    (loop [binding binding-vec]
+    (print-length-loop [binding binding-vec]
       (when (seq binding)
         (pprint-logical-block binding
           (write-out (first binding))
@@ -256,7 +256,7 @@
     (when (next alis)
       (.write ^java.io.Writer *out* " ")
       (pprint-newline :linear)
-     (loop [alis (next alis)]
+     (print-length-loop [alis (next alis)]
        (when alis
          (pprint-logical-block alis
           (write-out (first alis))
@@ -274,7 +274,7 @@
     (pprint-logical-block :prefix "(" :suffix ")"
       (pprint-indent :block 1)
       (apply (formatter-out "~w ~@_~w ~@_~w ~_") alis)
-      (loop [alis (seq (drop 3 alis))]
+      (print-length-loop [alis (seq (drop 3 alis))]
         (when alis
           (pprint-logical-block alis
             (write-out (first alis))
@@ -289,7 +289,7 @@
     (pprint-simple-code-list alis)))
 
 ;;; The map of symbols that are defined in an enclosing #() anonymous function
-(def ^{:private true} *symbol-map* {})
+(def ^:dynamic ^{:private true} *symbol-map* {})
 
 (defn- pprint-anon-func [alis]
   (let [args (second alis)
@@ -316,7 +316,7 @@
 (defn- pprint-simple-code-list [alis]
   (pprint-logical-block :prefix "(" :suffix ")"
     (pprint-indent :block 1)
-    (loop [alis (seq alis)]
+    (print-length-loop [alis (seq alis)]
       (when alis
 	(write-out (first alis))
 	(when (next alis)
@@ -342,7 +342,7 @@
                     %))
                amap))))
 
-(def ^{:private true} *code-table*
+(def ^:dynamic ^{:private true} *code-table*
      (two-forms
       (add-core-ns
        {'def pprint-hold-first, 'defonce pprint-hold-first, 
