@@ -96,6 +96,8 @@
 ;;; Functions to write tokens in the output buffer
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(def ^:private pp-newline (memoize #(System/getProperty "line.separator")))
+
 (declare emit-nl)
 
 (defmulti ^{:private true} write-token #(:type-tag %2))
@@ -223,7 +225,7 @@
            (recur (:parent lb)))))))
 
 (defn- emit-nl [^Writer this nl]
-  (.write (getf :base) (int \newline))
+  (.write (getf :base) (pp-newline))
   (dosync (setf :trailing-white-space nil))
   (let [lb (:logical-block nl)
         ^String prefix (:per-line-prefix lb)] 
@@ -310,6 +312,13 @@
       (write-tokens this buf true)
       (setf :buffer []))))
 
+(defn- write-white-space [^Writer this]
+  (when-let [^String tws (getf :trailing-white-space)]
+    ; (prlabel wws (str "*" tws "*"))
+    (.write (getf :base) tws)
+    (dosync
+     (setf :trailing-white-space nil))))
+
 ;;; If there are newlines in the string, print the lines up until the last newline, 
 ;;; making the appropriate adjustments. Return the remainder of the string
 (defn- write-initial-lines 
@@ -326,23 +335,19 @@
              (setf :pos newpos)
              (add-to-buffer this (make-buffer-blob l nil oldpos newpos))
              (write-buffered-output this))
-           (.write (getf :base) l))
+           (do
+             (write-white-space this)
+             (.write (getf :base) l)))
          (.write (getf :base) (int \newline))
          (doseq [^String l (next (butlast lines))]
            (.write (getf :base) l)
-           (.write (getf :base) (int \newline))
+           (.write (getf :base) (pp-newline))
            (if prefix
              (.write (getf :base) prefix)))
          (setf :buffering :writing)
          (last lines))))))
 
 
-(defn- write-white-space [^Writer this]
-  (if-let [^String tws (getf :trailing-white-space)]
-    (dosync
-     (.write (getf :base) tws)
-     (setf :trailing-white-space nil))))
-
 (defn- p-write-char [^Writer this ^Integer c]
   (if (= (getf :mode) :writing)
     (do 
@@ -399,6 +404,8 @@
                    (add-to-buffer this (make-buffer-blob s white-space oldpos newpos))))))
 
             Integer
+            (p-write-char this x)
+            Long
             (p-write-char this x))))
 
       (flush []
