@@ -18,7 +18,7 @@ import java.util.*;
 public abstract class APersistentVector extends AFn implements IPersistentVector, Iterable,
                                                                List,
                                                                RandomAccess, Comparable,
-                                                               Serializable {
+                                                               Serializable, IHashEq {
 int _hash = -1;
 
 public String toString(){
@@ -152,6 +152,17 @@ public int hashCode(){
 	return _hash;
 }
 
+public int hasheq(){
+	int hash = 1;
+	Iterator i = iterator();
+	while(i.hasNext())
+		{
+		Object obj = i.next();
+		hash = 31 * hash + Util.hasheq(obj);
+		}
+	return hash;
+}
+
 public Object get(int index){
 	return nth(index);
 }
@@ -244,7 +255,7 @@ public boolean addAll(int i, Collection c){
 }
 
 
-public Object invoke(Object arg1) throws Exception{
+public Object invoke(Object arg1) {
 	if(Util.isInteger(arg1))
 		return nth(((Number) arg1).intValue());
 	throw new IllegalArgumentException("Key must be integer");
@@ -355,19 +366,7 @@ public boolean containsAll(Collection c){
 }
 
 public Object[] toArray(Object[] a){
-	if(a.length >= count())
-		{
-		ISeq s = seq();
-		for(int i = 0; s != null; ++i, s = s.next())
-			{
-			a[i] = s.first();
-			}
-		if(a.length > count())
-			a[count()] = null;
-		return a;
-		}
-	else
-		return toArray();
+    return RT.seqToPassedArray(seq(), a);
 }
 
 public int size(){
@@ -445,14 +444,14 @@ public int compareTo(Object o){
 		return new APersistentVector.Seq(meta, v, i);
 	}
 
-	public Object reduce(IFn f) throws Exception{
+	public Object reduce(IFn f) {
 		Object ret = v.nth(i);
 		for(int x = i + 1; x < v.count(); x++)
 			ret = f.invoke(ret, v.nth(x));
 		return ret;
 	}
 
-	public Object reduce(IFn f, Object start) throws Exception{
+	public Object reduce(IFn f, Object start) {
 		Object ret = f.invoke(start, v.nth(i));
 		for(int x = i + 1; x < v.count(); x++)
 			ret = f.invoke(ret, v.nth(x));
