@@ -16,5 +16,5 @@ public interface IChunk extends Indexed{
 
 IChunk dropFirst();
 
-Object reduce(IFn f, Object start) throws Exception;
+Object reduce(IFn f, Object start) ;
 }
