@@ -10,34 +10,50 @@
 
 package clojure.lang;
 
-import java.io.*;
-import java.util.regex.Pattern;
-import java.util.regex.Matcher;
+import java.io.IOException;
+import java.io.PushbackReader;
+import java.io.Reader;
+import java.lang.Character;
+import java.lang.Class;
+import java.lang.Exception;
+import java.lang.IllegalArgumentException;
+import java.lang.IllegalStateException;
+import java.lang.Integer;
+import java.lang.Number;
+import java.lang.NumberFormatException;
+import java.lang.Object;
+import java.lang.RuntimeException;
+import java.lang.String;
+import java.lang.StringBuilder;
+import java.lang.Throwable;
+import java.lang.UnsupportedOperationException;
+import java.lang.reflect.Constructor;
+import java.math.BigDecimal;
+import java.math.BigInteger;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
-import java.math.BigInteger;
-import java.math.BigDecimal;
-import java.lang.*;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 public class LispReader{
 
-static final Symbol QUOTE = Symbol.create("quote");
-static final Symbol THE_VAR = Symbol.create("var");
-//static Symbol SYNTAX_QUOTE = Symbol.create(null, "syntax-quote");
-static Symbol UNQUOTE = Symbol.create("clojure.core", "unquote");
-static Symbol UNQUOTE_SPLICING = Symbol.create("clojure.core", "unquote-splicing");
-static Symbol CONCAT = Symbol.create("clojure.core", "concat");
-static Symbol SEQ = Symbol.create("clojure.core", "seq");
-static Symbol LIST = Symbol.create("clojure.core", "list");
-static Symbol APPLY = Symbol.create("clojure.core", "apply");
-static Symbol HASHMAP = Symbol.create("clojure.core", "hash-map");
-static Symbol HASHSET = Symbol.create("clojure.core", "hash-set");
-static Symbol VECTOR = Symbol.create("clojure.core", "vector");
-static Symbol WITH_META = Symbol.create("clojure.core", "with-meta");
-static Symbol META = Symbol.create("clojure.core", "meta");
-static Symbol DEREF = Symbol.create("clojure.core", "deref");
-//static Symbol DEREF_BANG = Symbol.create("clojure.core", "deref!");
+static final Symbol QUOTE = Symbol.intern("quote");
+static final Symbol THE_VAR = Symbol.intern("var");
+//static Symbol SYNTAX_QUOTE = Symbol.intern(null, "syntax-quote");
+static Symbol UNQUOTE = Symbol.intern("clojure.core", "unquote");
+static Symbol UNQUOTE_SPLICING = Symbol.intern("clojure.core", "unquote-splicing");
+static Symbol CONCAT = Symbol.intern("clojure.core", "concat");
+static Symbol SEQ = Symbol.intern("clojure.core", "seq");
+static Symbol LIST = Symbol.intern("clojure.core", "list");
+static Symbol APPLY = Symbol.intern("clojure.core", "apply");
+static Symbol HASHMAP = Symbol.intern("clojure.core", "hash-map");
+static Symbol HASHSET = Symbol.intern("clojure.core", "hash-set");
+static Symbol VECTOR = Symbol.intern("clojure.core", "vector");
+static Symbol WITH_META = Symbol.intern("clojure.core", "with-meta");
+static Symbol META = Symbol.intern("clojure.core", "meta");
+static Symbol DEREF = Symbol.intern("clojure.core", "deref");
+//static Symbol DEREF_BANG = Symbol.intern("clojure.core", "deref!");
 
 static IFn[] macros = new IFn[256];
 static IFn[] dispatchMacros = new IFn[256];
@@ -47,20 +63,21 @@ static Pattern symbolPat = Pattern.compile("[:]?([\\D&&[^/]].*/)?([\\D&&[^/]][^/
 //static Pattern intPat = Pattern.compile("[-+]?[0-9]+\\.?");
 static Pattern intPat =
 		Pattern.compile(
-				"([-+]?)(?:(0)|([1-9][0-9]*)|0[xX]([0-9A-Fa-f]+)|0([0-7]+)|([1-9][0-9]?)[rR]([0-9A-Za-z]+)|0[0-9]+)");
+				"([-+]?)(?:(0)|([1-9][0-9]*)|0[xX]([0-9A-Fa-f]+)|0([0-7]+)|([1-9][0-9]?)[rR]([0-9A-Za-z]+)|0[0-9]+)(N)?");
 static Pattern ratioPat = Pattern.compile("([-+]?[0-9]+)/([0-9]+)");
 static Pattern floatPat = Pattern.compile("([-+]?[0-9]+(\\.[0-9]*)?([eE][-+]?[0-9]+)?)(M)?");
-static final Symbol SLASH = Symbol.create("/");
-static final Symbol CLOJURE_SLASH = Symbol.create("clojure.core","/");
+static final Symbol SLASH = Symbol.intern("/");
+static final Symbol CLOJURE_SLASH = Symbol.intern("clojure.core","/");
 //static Pattern accessorPat = Pattern.compile("\\.[a-zA-Z_]\\w*");
 //static Pattern instanceMemberPat = Pattern.compile("\\.([a-zA-Z_][\\w\\.]*)\\.([a-zA-Z_]\\w*)");
 //static Pattern staticMemberPat = Pattern.compile("([a-zA-Z_][\\w\\.]*)\\.([a-zA-Z_]\\w*)");
 //static Pattern classNamePat = Pattern.compile("([a-zA-Z_][\\w\\.]*)\\.");
 
 //symbol->gensymbol
-static Var GENSYM_ENV = Var.create(null);
+static Var GENSYM_ENV = Var.create(null).setDynamic();
 //sorted-map num->gensymbol
-static Var ARG_ENV = Var.create(null);
+static Var ARG_ENV = Var.create(null).setDynamic();
+static IFn ctorReader = new CtorReader();
 
     static
 	{
@@ -98,12 +115,19 @@ static boolean isWhitespace(int ch){
 	return Character.isWhitespace(ch) || ch == ',';
 }
 
-static void unread(PushbackReader r, int ch) throws IOException{
+static void unread(PushbackReader r, int ch) {
 	if(ch != -1)
-		r.unread(ch);
+		try
+			{
+			r.unread(ch);
+			}
+		catch(IOException e)
+			{
+			throw Util.sneakyThrow(e);
+			}
 }
 
-public static class ReaderException extends Exception{
+public static class ReaderException extends RuntimeException{
 	final int line;
 
 	public ReaderException(int line, Throwable cause){
@@ -112,22 +136,33 @@ public static class ReaderException extends Exception{
 	}
 }
 
+static public int read1(Reader r){
+	try
+		{
+		return r.read();
+		}
+	catch(IOException e)
+		{
+		throw Util.sneakyThrow(e);
+		}
+}
+
 static public Object read(PushbackReader r, boolean eofIsError, Object eofValue, boolean isRecursive)
-		throws Exception{
+		{
 
 	try
 		{
 		for(; ;)
 			{
-			int ch = r.read();
+			int ch = read1(r);
 
 			while(isWhitespace(ch))
-				ch = r.read();
+				ch = read1(r);
 
 			if(ch == -1)
 				{
 				if(eofIsError)
-					throw new Exception("EOF while reading");
+					throw Util.runtimeException("EOF while reading");
 				return eofValue;
 				}
 
@@ -153,7 +188,7 @@ static public Object read(PushbackReader r, boolean eofIsError, Object eofValue,
 
 			if(ch == '+' || ch == '-')
 				{
-				int ch2 = r.read();
+				int ch2 = read1(r);
 				if(Character.isDigit(ch2))
 					{
 					unread(r, ch2);
@@ -174,20 +209,20 @@ static public Object read(PushbackReader r, boolean eofIsError, Object eofValue,
 	catch(Exception e)
 		{
 		if(isRecursive || !(r instanceof LineNumberingPushbackReader))
-			throw e;
+			throw Util.sneakyThrow(e);
 		LineNumberingPushbackReader rdr = (LineNumberingPushbackReader) r;
-		//throw new Exception(String.format("ReaderError:(%d,1) %s", rdr.getLineNumber(), e.getMessage()), e);
+		//throw Util.runtimeException(String.format("ReaderError:(%d,1) %s", rdr.getLineNumber(), e.getMessage()), e);
 		throw new ReaderException(rdr.getLineNumber(), e);
 		}
 }
 
-static private String readToken(PushbackReader r, char initch) throws Exception{
+static private String readToken(PushbackReader r, char initch) {
 	StringBuilder sb = new StringBuilder();
 	sb.append(initch);
 
 	for(; ;)
 		{
-		int ch = r.read();
+		int ch = read1(r);
 		if(ch == -1 || isWhitespace(ch) || isTerminatingMacro(ch))
 			{
 			unread(r, ch);
@@ -197,13 +232,13 @@ static private String readToken(PushbackReader r, char initch) throws Exception{
 		}
 }
 
-static private Object readNumber(PushbackReader r, char initch) throws Exception{
+static private Object readNumber(PushbackReader r, char initch) {
 	StringBuilder sb = new StringBuilder();
 	sb.append(initch);
 
 	for(; ;)
 		{
-		int ch = r.read();
+		int ch = read1(r);
 		if(ch == -1 || isWhitespace(ch) || isMacro(ch))
 			{
 			unread(r, ch);
@@ -219,7 +254,7 @@ static private Object readNumber(PushbackReader r, char initch) throws Exception
 	return n;
 }
 
-static private int readUnicodeChar(String token, int offset, int length, int base) throws Exception{
+static private int readUnicodeChar(String token, int offset, int length, int base) {
 	if(token.length() != offset + length)
 		throw new IllegalArgumentException("Invalid unicode character: \\" + token);
 	int uc = 0;
@@ -227,20 +262,20 @@ static private int readUnicodeChar(String token, int offset, int length, int bas
 		{
 		int d = Character.digit(token.charAt(i), base);
 		if(d == -1)
-			throw new IllegalArgumentException("Invalid digit: " + (char) d);
+			throw new IllegalArgumentException("Invalid digit: " + token.charAt(i));
 		uc = uc * base + d;
 		}
 	return (char) uc;
 }
 
-static private int readUnicodeChar(PushbackReader r, int initch, int base, int length, boolean exact) throws Exception{
+static private int readUnicodeChar(PushbackReader r, int initch, int base, int length, boolean exact) {
 	int uc = Character.digit(initch, base);
 	if(uc == -1)
-		throw new IllegalArgumentException("Invalid digit: " + initch);
+		throw new IllegalArgumentException("Invalid digit: " + (char) initch);
 	int i = 1;
 	for(; i < length; ++i)
 		{
-		int ch = r.read();
+		int ch = read1(r);
 		if(ch == -1 || isWhitespace(ch) || isMacro(ch))
 			{
 			unread(r, ch);
@@ -256,7 +291,7 @@ static private int readUnicodeChar(PushbackReader r, int initch, int base, int l
 	return uc;
 }
 
-static private Object interpretToken(String s) throws Exception{
+static private Object interpretToken(String s) {
 	if(s.equals("nil"))
 		{
 		return null;
@@ -283,7 +318,7 @@ static private Object interpretToken(String s) throws Exception{
 	if(ret != null)
 		return ret;
 
-	throw new Exception("Invalid token: " + s);
+	throw Util.runtimeException("Invalid token: " + s);
 }
 
 
@@ -327,7 +362,11 @@ private static Object matchNumber(String s){
 	if(m.matches())
 		{
 		if(m.group(2) != null)
-			return 0;
+			{
+			if(m.group(8) != null)
+				return BigInt.ZERO;
+			return Numbers.num(0);
+			}
 		boolean negate = (m.group(1).equals("-"));
 		String n;
 		int radix = 10;
@@ -342,7 +381,13 @@ private static Object matchNumber(String s){
 		if(n == null)
 			return null;
 		BigInteger bn = new BigInteger(n, radix);
-		return Numbers.reduce(negate ? bn.negate() : bn);
+		if(negate)
+			bn = bn.negate();
+		if(m.group(8) != null)
+			return BigInt.fromBigInteger(bn);
+		return bn.bitLength() < 64 ?
+		        Numbers.num(bn.longValue())
+				: BigInt.fromBigInteger(bn);
 		}
 	m = floatPat.matcher(s);
 	if(m.matches())
@@ -354,7 +399,8 @@ private static Object matchNumber(String s){
 	m = ratioPat.matcher(s);
 	if(m.matches())
 		{
-		return Numbers.divide(new BigInteger(m.group(1)), new BigInteger(m.group(2)));
+		return Numbers.divide(Numbers.reduceBigInt(BigInt.fromBigInteger(new BigInteger(m.group(1)))),
+		                      Numbers.reduceBigInt(BigInt.fromBigInteger(new BigInteger(m.group(2)))));
 		}
 	return null;
 }
@@ -370,25 +416,25 @@ static private boolean isMacro(int ch){
 }
 
 static private boolean isTerminatingMacro(int ch){
-	return (ch != '#' && ch < macros.length && macros[ch] != null);
+	return (ch != '#' && ch != '\'' && isMacro(ch));
 }
 
 public static class RegexReader extends AFn{
 	static StringReader stringrdr = new StringReader();
 
-	public Object invoke(Object reader, Object doublequote) throws Exception{
+	public Object invoke(Object reader, Object doublequote) {
 		StringBuilder sb = new StringBuilder();
 		Reader r = (Reader) reader;
-		for(int ch = r.read(); ch != '"'; ch = r.read())
+		for(int ch = read1(r); ch != '"'; ch = read1(r))
 			{
 			if(ch == -1)
-				throw new Exception("EOF while reading regex");
+				throw Util.runtimeException("EOF while reading regex");
 			sb.append( (char) ch );
 			if(ch == '\\')	//escape
 				{
-				ch = r.read();
+				ch = read1(r);
 				if(ch == -1)
-					throw new Exception("EOF while reading regex");
+					throw Util.runtimeException("EOF while reading regex");
 				sb.append( (char) ch ) ;
 				}
 			}
@@ -397,19 +443,19 @@ public static class RegexReader extends AFn{
 }
 
 public static class StringReader extends AFn{
-	public Object invoke(Object reader, Object doublequote) throws Exception{
+	public Object invoke(Object reader, Object doublequote) {
 		StringBuilder sb = new StringBuilder();
 		Reader r = (Reader) reader;
 
-		for(int ch = r.read(); ch != '"'; ch = r.read())
+		for(int ch = read1(r); ch != '"'; ch = read1(r))
 			{
 			if(ch == -1)
-				throw new Exception("EOF while reading string");
+				throw Util.runtimeException("EOF while reading string");
 			if(ch == '\\')	//escape
 				{
-				ch = r.read();
+				ch = read1(r);
 				if(ch == -1)
-					throw new Exception("EOF while reading string");
+					throw Util.runtimeException("EOF while reading string");
 				switch(ch)
 					{
 					case 't':
@@ -433,9 +479,9 @@ public static class StringReader extends AFn{
 						break;
 					case 'u':
 					{
-					ch = r.read();
+					ch = read1(r);
 					if (Character.digit(ch, 16) == -1)
-					    throw new Exception("Invalid unicode escape: \\u" + (char) ch);
+					    throw Util.runtimeException("Invalid unicode escape: \\u" + (char) ch);
 					ch = readUnicodeChar((PushbackReader) r, ch, 16, 4, true);
 					break;
 					}
@@ -445,10 +491,10 @@ public static class StringReader extends AFn{
 						{
 						ch = readUnicodeChar((PushbackReader) r, ch, 8, 3, false);
 						if(ch > 0377)
-							throw new Exception("Octal escape sequence must be in range [0, 377].");
+							throw Util.runtimeException("Octal escape sequence must be in range [0, 377].");
 						}
 					else
-						throw new Exception("Unsupported escape character: \\" + (char) ch);
+						throw Util.runtimeException("Unsupported escape character: \\" + (char) ch);
 					}
 					}
 				}
@@ -459,12 +505,12 @@ public static class StringReader extends AFn{
 }
 
 public static class CommentReader extends AFn{
-	public Object invoke(Object reader, Object semicolon) throws Exception{
+	public Object invoke(Object reader, Object semicolon) {
 		Reader r = (Reader) reader;
 		int ch;
 		do
 			{
-			ch = r.read();
+			ch = read1(r);
 			} while(ch != -1 && ch != '\n' && ch != '\r');
 		return r;
 	}
@@ -472,7 +518,7 @@ public static class CommentReader extends AFn{
 }
 
 public static class DiscardReader extends AFn{
-	public Object invoke(Object reader, Object underscore) throws Exception{
+	public Object invoke(Object reader, Object underscore) {
 		PushbackReader r = (PushbackReader) reader;
 		read(r, true, null, true);
 		return r;
@@ -486,7 +532,7 @@ public static class WrappingReader extends AFn{
 		this.sym = sym;
 	}
 
-	public Object invoke(Object reader, Object quote) throws Exception{
+	public Object invoke(Object reader, Object quote) {
 		PushbackReader r = (PushbackReader) reader;
 		Object o = read(r, true, null, true);
 		return RT.list(sym, o);
@@ -503,7 +549,7 @@ public static class DeprecatedWrappingReader extends AFn{
                 this.macro = macro;
 	}
 
-	public Object invoke(Object reader, Object quote) throws Exception{
+	public Object invoke(Object reader, Object quote) {
                 System.out.println("WARNING: reader macro " + macro +
                                    " is deprecated; use " + sym.getName() +
                                    " instead");
@@ -515,7 +561,7 @@ public static class DeprecatedWrappingReader extends AFn{
 }
 
 public static class VarReader extends AFn{
-	public Object invoke(Object reader, Object quote) throws Exception{
+	public Object invoke(Object reader, Object quote) {
 		PushbackReader r = (PushbackReader) reader;
 		Object o = read(r, true, null, true);
 //		if(o instanceof Symbol)
@@ -531,11 +577,11 @@ public static class VarReader extends AFn{
 /*
 static class DerefReader extends AFn{
 
-	public Object invoke(Object reader, Object quote) throws Exception{
+	public Object invoke(Object reader, Object quote) {
 		PushbackReader r = (PushbackReader) reader;
-		int ch = r.read();
+		int ch = read1(r);
 		if(ch == -1)
-			throw new Exception("EOF while reading character");
+			throw Util.runtimeException("EOF while reading character");
 		if(ch == '!')
 			{
 			Object o = read(r, true, null, true);
@@ -553,13 +599,22 @@ static class DerefReader extends AFn{
 */
 
 public static class DispatchReader extends AFn{
-	public Object invoke(Object reader, Object hash) throws Exception{
-		int ch = ((Reader) reader).read();
+	public Object invoke(Object reader, Object hash) {
+		int ch = read1((Reader) reader);
 		if(ch == -1)
-			throw new Exception("EOF while reading character");
+			throw Util.runtimeException("EOF while reading character");
 		IFn fn = dispatchMacros[ch];
-		if(fn == null)
-			throw new Exception(String.format("No dispatch macro for: %c", (char) ch));
+
+		// Try the ctor reader first
+		if(fn == null) {
+			unread((PushbackReader) reader, ch);
+			Object result = ctorReader.invoke(reader, ch);
+
+			if(result != null)
+				return result;
+			else
+				throw Util.runtimeException(String.format("No dispatch macro for: %c", (char) ch));
+		}
 		return fn.invoke(reader, ch);
 	}
 }
@@ -569,7 +624,7 @@ static Symbol garg(int n){
 }
 
 public static class FnReader extends AFn{
-	public Object invoke(Object reader, Object lparen) throws Exception{
+	public Object invoke(Object reader, Object lparen) {
 		PushbackReader r = (PushbackReader) reader;
 		if(ARG_ENV.deref() != null)
 			throw new IllegalStateException("Nested #()s are not allowed");
@@ -577,7 +632,7 @@ public static class FnReader extends AFn{
 			{
 			Var.pushThreadBindings(
 					RT.map(ARG_ENV, PersistentTreeMap.EMPTY));
-			r.unread('(');
+			unread(r, '(');
 			Object form = read(r, true, null, true);
 
 			PersistentVector args = PersistentVector.EMPTY;
@@ -628,13 +683,13 @@ static Symbol registerArg(int n){
 }
 
 static class ArgReader extends AFn{
-	public Object invoke(Object reader, Object pct) throws Exception{
+	public Object invoke(Object reader, Object pct) {
 		PushbackReader r = (PushbackReader) reader;
 		if(ARG_ENV.deref() == null)
 			{
 			return interpretToken(readToken(r, '%'));
 			}
-		int ch = r.read();
+		int ch = read1(r);
 		unread(r, ch);
 		//% alone is first arg
 		if(ch == -1 || isWhitespace(ch) || isTerminatingMacro(ch))
@@ -651,14 +706,16 @@ static class ArgReader extends AFn{
 }
 
 public static class MetaReader extends AFn{
-	public Object invoke(Object reader, Object caret) throws Exception{
+	public Object invoke(Object reader, Object caret) {
 		PushbackReader r = (PushbackReader) reader;
 		int line = -1;
 		if(r instanceof LineNumberingPushbackReader)
 			line = ((LineNumberingPushbackReader) r).getLineNumber();
 		Object meta = read(r, true, null, true);
-		if(meta instanceof Symbol || meta instanceof Keyword || meta instanceof String)
+		if(meta instanceof Symbol || meta instanceof String)
 			meta = RT.map(RT.TAG_KEY, meta);
+		else if (meta instanceof Keyword)
+			meta = RT.map(meta, RT.T);
 		else if(!(meta instanceof IPersistentMap))
 			throw new IllegalArgumentException("Metadata must be Symbol,Keyword,String or Map");
 
@@ -672,7 +729,12 @@ public static class MetaReader extends AFn{
 				((IReference)o).resetMeta((IPersistentMap) meta);
 				return o;
 				}
-			return ((IObj) o).withMeta((IPersistentMap) meta);
+			Object ometa = RT.meta(o);
+			for(ISeq s = RT.seq(meta); s != null; s = s.next()) {
+				IMapEntry kv = (IMapEntry) s.first();
+				ometa = RT.assoc(ometa, kv.getKey(), kv.getValue());
+				}
+			return ((IObj) o).withMeta((IPersistentMap) ometa);
 			}
 		else
 			throw new IllegalArgumentException("Metadata can only be applied to IMetas");
@@ -681,7 +743,7 @@ public static class MetaReader extends AFn{
 }
 
 public static class SyntaxQuoteReader extends AFn{
-	public Object invoke(Object reader, Object backquote) throws Exception{
+	public Object invoke(Object reader, Object backquote) {
 		PushbackReader r = (PushbackReader) reader;
 		try
 			{
@@ -697,7 +759,7 @@ public static class SyntaxQuoteReader extends AFn{
 			}
 	}
 
-	static Object syntaxQuote(Object form) throws Exception{
+	static Object syntaxQuote(Object form) {
 		Object ret;
 		if(Compiler.isSpecial(form))
 			ret = RT.list(Compiler.QUOTE, form);
@@ -791,7 +853,7 @@ public static class SyntaxQuoteReader extends AFn{
 		return ret;
 	}
 
-	private static ISeq sqExpandList(ISeq seq) throws Exception{
+	private static ISeq sqExpandList(ISeq seq) {
 		PersistentVector ret = PersistentVector.EMPTY;
 		for(; seq != null; seq = seq.next())
 			{
@@ -828,11 +890,11 @@ static boolean isUnquote(Object form){
 }
 
 static class UnquoteReader extends AFn{
-	public Object invoke(Object reader, Object comma) throws Exception{
+	public Object invoke(Object reader, Object comma) {
 		PushbackReader r = (PushbackReader) reader;
-		int ch = r.read();
+		int ch = read1(r);
 		if(ch == -1)
-			throw new Exception("EOF while reading character");
+			throw Util.runtimeException("EOF while reading character");
 		if(ch == '@')
 			{
 			Object o = read(r, true, null, true);
@@ -849,11 +911,11 @@ static class UnquoteReader extends AFn{
 }
 
 public static class CharacterReader extends AFn{
-	public Object invoke(Object reader, Object backslash) throws Exception{
+	public Object invoke(Object reader, Object backslash) {
 		PushbackReader r = (PushbackReader) reader;
-		int ch = r.read();
+		int ch = read1(r);
 		if(ch == -1)
-			throw new Exception("EOF while reading character");
+			throw Util.runtimeException("EOF while reading character");
 		String token = readToken(r, (char) ch);
 		if(token.length() == 1)
 			return Character.valueOf(token.charAt(0));
@@ -873,26 +935,26 @@ public static class CharacterReader extends AFn{
 		    {
 			 char c = (char) readUnicodeChar(token, 1, 4, 16);
 			 if(c >= '\uD800' && c <= '\uDFFF') // surrogate code unit?
-			     throw new Exception("Invalid character constant: \\u" + Integer.toString(c, 16));
+			     throw Util.runtimeException("Invalid character constant: \\u" + Integer.toString(c, 16));
 			 return c;
 		    }
 		else if(token.startsWith("o"))
 			{
 			int len = token.length() - 1;
 			if(len > 3)
-				throw new Exception("Invalid octal escape sequence length: " + len);
+				throw Util.runtimeException("Invalid octal escape sequence length: " + len);
 			int uc = readUnicodeChar(token, 1, len, 8);
 			if(uc > 0377)
-				throw new Exception("Octal escape sequence must be in range [0, 377].");
+				throw Util.runtimeException("Octal escape sequence must be in range [0, 377].");
 			return (char) uc;
 			}
-		throw new Exception("Unsupported character: \\" + token);
+		throw Util.runtimeException("Unsupported character: \\" + token);
 	}
 
 }
 
 public static class ListReader extends AFn{
-	public Object invoke(Object reader, Object leftparen) throws Exception{
+	public Object invoke(Object reader, Object leftparen) {
 		PushbackReader r = (PushbackReader) reader;
 		int line = -1;
 		if(r instanceof LineNumberingPushbackReader)
@@ -910,17 +972,18 @@ public static class ListReader extends AFn{
 
 }
 
+/*
 static class CtorReader extends AFn{
-	static final Symbol cls = Symbol.create("class");
+	static final Symbol cls = Symbol.intern("class");
 
-	public Object invoke(Object reader, Object leftangle) throws Exception{
+	public Object invoke(Object reader, Object leftangle) {
 		PushbackReader r = (PushbackReader) reader;
 		// #<class classname>
 		// #<classname args*>
 		// #<classname/staticMethod args*>
 		List list = readDelimitedList('>', r, true);
 		if(list.isEmpty())
-			throw new Exception("Must supply 'class', classname or classname/staticMethod");
+			throw Util.runtimeException("Must supply 'class', classname or classname/staticMethod");
 		Symbol s = (Symbol) list.get(0);
 		Object[] args = list.subList(1, list.size()).toArray();
 		if(s.equals(cls))
@@ -938,14 +1001,14 @@ static class CtorReader extends AFn{
 			return Reflector.invokeConstructor(RT.classForName(s.name), args);
 			}
 	}
-
 }
+*/
 
 public static class EvalReader extends AFn{
-	public Object invoke(Object reader, Object eq) throws Exception{
+	public Object invoke(Object reader, Object eq) {
 		if (!RT.booleanCast(RT.READEVAL.deref()))
 	    {
-		  throw new Exception("EvalReader not allowed when *read-eval* is false.");
+		  throw Util.runtimeException("EvalReader not allowed when *read-eval* is false.");
 	    }
 		
 		PushbackReader r = (PushbackReader) reader;
@@ -977,7 +1040,7 @@ public static class EvalReader extends AFn{
 				{
 				return ((IFn) v).applyTo(RT.next(o));
 				}
-			throw new Exception("Can't resolve " + fs);
+			throw Util.runtimeException("Can't resolve " + fs);
 			}
 		else
 			throw new IllegalArgumentException("Unsupported #= form");
@@ -985,7 +1048,7 @@ public static class EvalReader extends AFn{
 }
 
 //static class ArgVectorReader extends AFn{
-//	public Object invoke(Object reader, Object leftparen) throws Exception{
+//	public Object invoke(Object reader, Object leftparen) {
 //		PushbackReader r = (PushbackReader) reader;
 //		return ArgVector.create(readDelimitedList('|', r, true));
 //	}
@@ -993,7 +1056,7 @@ public static class EvalReader extends AFn{
 //}
 
 public static class VectorReader extends AFn{
-	public Object invoke(Object reader, Object leftparen) throws Exception{
+	public Object invoke(Object reader, Object leftparen) {
 		PushbackReader r = (PushbackReader) reader;
 		return LazilyPersistentVector.create(readDelimitedList(']', r, true));
 	}
@@ -1001,15 +1064,18 @@ public static class VectorReader extends AFn{
 }
 
 public static class MapReader extends AFn{
-	public Object invoke(Object reader, Object leftparen) throws Exception{
+	public Object invoke(Object reader, Object leftparen) {
 		PushbackReader r = (PushbackReader) reader;
-		return RT.map(readDelimitedList('}', r, true).toArray());
+		Object[] a = readDelimitedList('}', r, true).toArray();
+		if((a.length & 1) == 1)
+			throw Util.runtimeException("Map literal must contain an even number of forms");
+		return RT.map(a);
 	}
 
 }
 
 public static class SetReader extends AFn{
-	public Object invoke(Object reader, Object leftbracket) throws Exception{
+	public Object invoke(Object reader, Object leftbracket) {
 		PushbackReader r = (PushbackReader) reader;
 		return PersistentHashSet.createWithCheck(readDelimitedList('}', r, true));
 	}
@@ -1017,30 +1083,39 @@ public static class SetReader extends AFn{
 }
 
 public static class UnmatchedDelimiterReader extends AFn{
-	public Object invoke(Object reader, Object rightdelim) throws Exception{
-		throw new Exception("Unmatched delimiter: " + rightdelim);
+	public Object invoke(Object reader, Object rightdelim) {
+		throw Util.runtimeException("Unmatched delimiter: " + rightdelim);
 	}
 
 }
 
 public static class UnreadableReader extends AFn{
-	public Object invoke(Object reader, Object leftangle) throws Exception{
-		throw new Exception("Unreadable form");
+	public Object invoke(Object reader, Object leftangle) {
+		throw Util.runtimeException("Unreadable form");
 	}
 }
 
-public static List readDelimitedList(char delim, PushbackReader r, boolean isRecursive) throws Exception{
+public static List readDelimitedList(char delim, PushbackReader r, boolean isRecursive) {
+	final int firstline =
+		(r instanceof LineNumberingPushbackReader) ?
+		((LineNumberingPushbackReader) r).getLineNumber() : -1;
+
 	ArrayList a = new ArrayList();
 
 	for(; ;)
 		{
-		int ch = r.read();
+		int ch = read1(r);
 
 		while(isWhitespace(ch))
-			ch = r.read();
+			ch = read1(r);
 
 		if(ch == -1)
-			throw new Exception("EOF while reading");
+			{
+			if(firstline < 0)
+				throw Util.runtimeException("EOF while reading");
+			else
+				throw Util.runtimeException("EOF while reading, starting at line " + firstline);
+			}
 
 		if(ch == delim)
 			break;
@@ -1067,6 +1142,84 @@ public static List readDelimitedList(char delim, PushbackReader r, boolean isRec
 	return a;
 }
 
+public static class CtorReader extends AFn{
+	public Object invoke(Object reader, Object firstChar){
+		PushbackReader r = (PushbackReader) reader;
+		Object name = read(r, true, null, false);
+                if (!(name instanceof Symbol)) 
+                        throw new RuntimeException("Reader tag must be a symbol");
+		Symbol sym = (Symbol)name;
+		return sym.getName().contains(".") ? readRecord(r, sym) : readTagged(r, sym);
+	}
+
+	private Object readTagged(PushbackReader reader, Symbol tag){
+		Object o = read(reader, true, null, true);
+
+		ILookup data_readers = (ILookup)RT.DATA_READERS.deref();
+		IFn data_reader = (IFn)RT.get(data_readers, tag);
+		if(data_reader == null){
+                        data_readers = (ILookup)RT.DEFAULT_DATA_READERS.deref();
+                        data_reader = (IFn)RT.get(data_readers, tag);
+                        if(data_reader == null)
+                                throw new RuntimeException("No reader function for tag " + tag.toString());
+                }
+
+                return data_reader.invoke(o);
+	}
+
+        private Object readRecord(PushbackReader r, Symbol recordName){
+		Class recordClass = RT.classForName(recordName.toString());
+		char endch;
+		boolean shortForm = true;
+		int ch = read1(r);
+
+		// flush whitespace
+		//while(isWhitespace(ch))
+		//	ch = read1(r);
+
+		// A defrecord ctor can take two forms. Check for map->R version first.
+		if(ch == '{')
+			{
+				endch = '}';
+				shortForm = false;
+			}
+		else if (ch == '[')
+			endch = ']';
+		else
+			throw Util.runtimeException("Unreadable constructor form starting with \"#" + recordName + (char) ch + "\"");
+
+		Object[] recordEntries = readDelimitedList(endch, r, true).toArray();
+		Object ret = null;
+		Constructor[] allctors = ((Class)recordClass).getConstructors();
+
+		if(shortForm)
+			{
+			boolean ctorFound = false;
+			for (Constructor ctor : allctors)
+				if(ctor.getParameterTypes().length == recordEntries.length)
+					ctorFound = true;
+
+			if(!ctorFound)
+				throw Util.runtimeException("Unexpected number of constructor arguments to " + recordClass.toString() + ": got " + recordEntries.length);
+
+			ret = Reflector.invokeConstructor(recordClass, recordEntries);
+			}
+		else
+			{
+
+			IPersistentMap vals = RT.map(recordEntries);
+			for(ISeq s = RT.keys(vals); s != null; s = s.next())
+				{
+				if(!(s.first() instanceof Keyword))
+					throw Util.runtimeException("Unreadable defrecord form: key must be of type clojure.lang.Keyword, got " + s.first().toString());
+				}
+			ret = Reflector.invokeStaticMethod(recordClass, "create", new Object[]{vals});
+			}
+
+	return ret;
+	}
+}
+
 /*
 public static void main(String[] args) throws Exception{
 	//RT.init();
