@@ -117,16 +117,20 @@ public PersistentHashMap(IPersistentMap meta, int count, INode root, boolean has
 	this.nullValue = nullValue;
 }
 
+static int hash(Object k){
+	return Util.hasheq(k);
+}
+
 public boolean containsKey(Object key){
 	if(key == null)
 		return hasNull;
-	return (root != null) ? root.find(0, Util.hash(key), key, NOT_FOUND) != NOT_FOUND : false;
+	return (root != null) ? root.find(0, hash(key), key, NOT_FOUND) != NOT_FOUND : false;
 }
 
 public IMapEntry entryAt(Object key){
 	if(key == null)
 		return hasNull ? new MapEntry(null, nullValue) : null;
-	return (root != null) ? root.find(0, Util.hash(key), key) : null;
+	return (root != null) ? root.find(0, hash(key), key) : null;
 }
 
 public IPersistentMap assoc(Object key, Object val){
@@ -137,7 +141,7 @@ public IPersistentMap assoc(Object key, Object val){
 	}
 	Box addedLeaf = new Box(null);
 	INode newroot = (root == null ? BitmapIndexedNode.EMPTY : root) 
-			.assoc(0, Util.hash(key), key, val, addedLeaf);
+			.assoc(0, hash(key), key, val, addedLeaf);
 	if(newroot == root)
 		return this;
 	return new PersistentHashMap(meta(), addedLeaf.val == null ? count : count + 1, newroot, hasNull, nullValue);
@@ -146,7 +150,7 @@ public IPersistentMap assoc(Object key, Object val){
 public Object valAt(Object key, Object notFound){
 	if(key == null)
 		return hasNull ? nullValue : notFound;
-	return root != null ? root.find(0, Util.hash(key), key, notFound) : notFound;
+	return root != null ? root.find(0, hash(key), key, notFound) : notFound;
 }
 
 public Object valAt(Object key){
@@ -164,7 +168,7 @@ public IPersistentMap without(Object key){
 		return hasNull ? new PersistentHashMap(meta(), count - 1, root, false, null) : this;
 	if(root == null)
 		return this;
-	INode newroot = root.without(0, Util.hash(key), key);
+	INode newroot = root.without(0, hash(key), key);
 	if(newroot == root)
 		return this;
 	return new PersistentHashMap(meta(), count - 1, newroot, hasNull, nullValue); 
@@ -174,6 +178,14 @@ public Iterator iterator(){
 	return new SeqIterator(seq());
 }
 
+public Object kvreduce(IFn f, Object init){
+    init = hasNull?f.invoke(init,null,nullValue):init;
+    if(root != null){
+        return root.kvreduce(f,init);
+    }
+    return init;
+}
+
 public int count(){
 	return count;
 }
@@ -238,7 +250,7 @@ static final class TransientHashMap extends ATransientMap {
 //		Box leafFlag = new Box(null);
 		leafFlag.val = null;
 		INode n = (root == null ? BitmapIndexedNode.EMPTY : root)
-			.assoc(edit, 0, Util.hash(key), key, val, leafFlag);
+			.assoc(edit, 0, hash(key), key, val, leafFlag);
 		if (n != this.root)
 			this.root = n; 
 		if(leafFlag.val != null) this.count++;
@@ -256,7 +268,7 @@ static final class TransientHashMap extends ATransientMap {
 		if (root == null) return this;
 //		Box leafFlag = new Box(null);
 		leafFlag.val = null;
-		INode n = root.without(edit, 0, Util.hash(key), key, leafFlag);
+		INode n = root.without(edit, 0, hash(key), key, leafFlag);
 		if (n != root)
 			this.root = n;
 		if(leafFlag.val != null) this.count--;
@@ -276,7 +288,7 @@ static final class TransientHashMap extends ATransientMap {
 				return notFound;
 		if (root == null)
 			return null;
-		return root.find(0, Util.hash(key), key, notFound);
+		return root.find(0, hash(key), key, notFound);
 	}
 
 	int doCount() {
@@ -307,6 +319,9 @@ static interface INode extends Serializable {
 	INode assoc(AtomicReference<Thread> edit, int shift, int hash, Object key, Object val, Box addedLeaf);
 
 	INode without(AtomicReference<Thread> edit, int shift, int hash, Object key, Box removedLeaf);
+
+    public Object kvreduce(IFn f, Object init);
+
 }
 
 final static class ArrayNode implements INode{
@@ -367,6 +382,16 @@ final static class ArrayNode implements INode{
 		return Seq.create(array);
 	}
 
+    public Object kvreduce(IFn f, Object init){
+        for(INode node : array)
+            {
+            if(node != null)
+                init = node.kvreduce(f,init);
+            }
+        return init;
+    }
+
+
 	private ArrayNode ensureEditable(AtomicReference<Thread> edit){
 		if(this.edit == edit)
 			return this;
@@ -525,7 +550,7 @@ final static class BitmapIndexedNode implements INode{
 						if (array[j] == null)
 							nodes[i] = (INode) array[j+1];
 						else
-							nodes[i] = EMPTY.assoc(shift + 5, Util.hash(array[j]), array[j], array[j+1], addedLeaf);
+							nodes[i] = EMPTY.assoc(shift + 5, hash(array[j]), array[j], array[j+1], addedLeaf);
 						j += 2;
 					}
 				return new ArrayNode(null, n + 1, nodes);
@@ -596,6 +621,11 @@ final static class BitmapIndexedNode implements INode{
 		return NodeSeq.create(array);
 	}
 
+    public Object kvreduce(IFn f, Object init){
+         return NodeSeq.kvreduce(array,f,init);
+    }
+
+
 	private BitmapIndexedNode ensureEditable(AtomicReference<Thread> edit){
 		if(this.edit == edit)
 			return this;
@@ -670,7 +700,7 @@ final static class BitmapIndexedNode implements INode{
 						if (array[j] == null)
 							nodes[i] = (INode) array[j+1];
 						else
-							nodes[i] = EMPTY.assoc(edit, shift + 5, Util.hash(array[j]), array[j], array[j+1], addedLeaf);
+							nodes[i] = EMPTY.assoc(edit, shift + 5, hash(array[j]), array[j], array[j+1], addedLeaf);
 						j += 2;
 					}
 				return new ArrayNode(edit, n + 1, nodes);
@@ -704,7 +734,6 @@ final static class BitmapIndexedNode implements INode{
 				return editAndSet(edit, 2*idx+1, n); 
 			if (bitmap == bit) 
 				return null;
-			removedLeaf.val = removedLeaf;
 			return editAndRemovePair(edit, bit, idx); 
 		}
 		if(Util.equiv(key, keyOrNull)) {
@@ -781,6 +810,10 @@ final static class HashCollisionNode implements INode{
 		return NodeSeq.create(array);
 	}
 
+    public Object kvreduce(IFn f, Object init){
+         return NodeSeq.kvreduce(array,f,init);
+    }
+
 	public int findIndex(Object key){
 		for(int i = 0; i < 2*count; i+=2)
 			{
@@ -793,7 +826,9 @@ final static class HashCollisionNode implements INode{
 	private HashCollisionNode ensureEditable(AtomicReference<Thread> edit){
 		if(this.edit == edit)
 			return this;
-		return new HashCollisionNode(edit, hash, count, array);
+		Object[] newArray = new Object[2*(count+1)]; // make room for next assoc
+		System.arraycopy(array, 0, newArray, 0, 2*count);
+		return new HashCollisionNode(edit, hash, count, newArray);
 	}
 
 	private HashCollisionNode ensureEditable(AtomicReference<Thread> edit, int count, Object[] array){
@@ -849,6 +884,7 @@ final static class HashCollisionNode implements INode{
 		int idx = findIndex(key);
 		if(idx == -1)
 			return this;
+		removedLeaf.val = removedLeaf;
 		if(count == 1)
 			return null;
 		HashCollisionNode editable = ensureEditable(edit);
@@ -974,7 +1010,7 @@ private static Object[] removePair(Object[] array, int i) {
 }
 
 private static INode createNode(int shift, Object key1, Object val1, int key2hash, Object key2, Object val2) {
-	int key1hash = Util.hash(key1);
+	int key1hash = hash(key1);
 	if(key1hash == key2hash)
 		return new HashCollisionNode(null, key1hash, 2, new Object[] {key1, val1, key2, val2});
 	Box _ = new Box(null);
@@ -985,7 +1021,7 @@ private static INode createNode(int shift, Object key1, Object val1, int key2has
 }
 
 private static INode createNode(AtomicReference<Thread> edit, int shift, Object key1, Object val1, int key2hash, Object key2, Object val2) {
-	int key1hash = Util.hash(key1);
+	int key1hash = hash(key1);
 	if(key1hash == key2hash)
 		return new HashCollisionNode(null, key1hash, 2, new Object[] {key1, val1, key2, val2});
 	Box _ = new Box(null);
@@ -1011,6 +1047,21 @@ static final class NodeSeq extends ASeq {
 		return create(array, 0, null);
 	}
 
+    static public Object kvreduce(Object[] array, IFn f, Object init){
+         for(int i=0;i<array.length;i+=2)
+             {
+             if(array[i] != null)
+                 init = f.invoke(init, array[i], array[i+1]);
+             else
+                 {
+                 INode node = (INode) array[i+1];
+                 if(node != null)
+                     init = node.kvreduce(f,init);
+                 }
+             }
+        return init;
+    }
+
 	private static ISeq create(Object[] array, int i, ISeq s) {
 		if(s != null)
 			return new NodeSeq(null, array, i, s);
@@ -1051,4 +1102,4 @@ static final class NodeSeq extends ASeq {
 	}
 }
 
-}
\ No newline at end of file
+}
