@@ -182,6 +182,8 @@ final static Keyword TAG_KEY = Keyword.intern(null, "tag");
 final static Keyword CONST_KEY = Keyword.intern(null, "const");
 final static public Var AGENT = Var.intern(CLOJURE_NS, Symbol.intern("*agent*"), null).setDynamic();
 final static public Var READEVAL = Var.intern(CLOJURE_NS, Symbol.intern("*read-eval*"), T).setDynamic();
+final static public Var DATA_READERS = Var.intern(CLOJURE_NS, Symbol.intern("*data-readers*"), RT.map()).setDynamic();
+final static public Var DEFAULT_DATA_READERS = Var.intern(CLOJURE_NS, Symbol.intern("default-data-readers"), RT.map());
 final static public Var ASSERT = Var.intern(CLOJURE_NS, Symbol.intern("*assert*"), T).setDynamic();
 final static public Var MATH_CONTEXT = Var.intern(CLOJURE_NS, Symbol.intern("*math-context*"), null).setDynamic();
 static Keyword LINE_KEY = Keyword.intern(null, "line");
@@ -306,7 +308,7 @@ static{
 				               }
 			               catch(IOException e)
 				               {
-				               throw Util.runtimeException(e);
+				               throw Util.sneakyThrow(e);
 				               }
 		               }
 	               });
@@ -316,7 +318,7 @@ static{
 		doInit();
 	}
 	catch(Exception e) {
-		throw Util.runtimeException(e);
+		throw Util.sneakyThrow(e);
 	}
 }
 
@@ -456,6 +458,11 @@ static public int nextID(){
 	return id.getAndIncrement();
 }
 
+// Load a library in the System ClassLoader instead of Clojure's own.
+public static void loadLibrary(String libname){
+    System.loadLibrary(libname);
+}
+
 
 ////////////// Collections support /////////////////////////////////
 
@@ -695,6 +702,10 @@ static public Object contains(Object coll, Object key){
 		Map m = (Map) coll;
 		return m.containsKey(key) ? T : F;
 	}
+	else if(coll instanceof Set) {
+		Set s = (Set) coll;
+		return s.contains(key) ? T : F;
+	}
 	else if(key instanceof Number && (coll instanceof String || coll.getClass().isArray())) {
 		int n = ((Number) key).intValue();
 		return n >= 0 && n < count(coll);
@@ -1559,6 +1570,21 @@ static public Object[] seqToArray(ISeq seq){
 	return ret;
 }
 
+    // supports java Collection.toArray(T[])
+    static public Object[] seqToPassedArray(ISeq seq, Object[] passed){
+        Object[] dest = passed;
+        int len = count(seq);
+        if (len > dest.length) {
+            dest = (Object[]) Array.newInstance(passed.getClass().getComponentType(), len);
+        }
+        for(int i = 0; seq != null; ++i, seq = seq.next())
+            dest[i] = seq.first();
+        if (len < passed.length) {
+            dest[len] = null;
+        }
+        return dest;
+    }
+
 static public Object seqToTypedArray(ISeq seq) {
 	Class type = (seq != null) ? seq.first().getClass() : Object.class;
 	return seqToTypedArray(type, seq);
@@ -1671,7 +1697,7 @@ static public String printString(Object x){
 		return sw.toString();
 	}
 	catch(Exception e) {
-		throw Util.runtimeException(e);
+		throw Util.sneakyThrow(e);
 	}
 }
 
@@ -1681,7 +1707,7 @@ static public Object readString(String s){
 		return LispReader.read(r, true, null, false);
 	}
 	catch(Exception e) {
-		throw Util.runtimeException(e);
+		throw Util.sneakyThrow(e);
 	}
 }
 
@@ -2014,7 +2040,7 @@ static public Class classForName(String name) {
 		}
 	catch(ClassNotFoundException e)
 		{
-		throw Util.runtimeException(e);
+		throw Util.sneakyThrow(e);
 		}
 }
 
