@@ -19,10 +19,21 @@
   ;; too many contextual things for this kind of caching to work...
   (testing "classes are never cached, even if their bodies are the same"
     (is (= :b
-           (eval
+          (eval
             '(do
                (defmacro my-macro [] :a)
                (defn do-macro [] (my-macro))
                (defmacro my-macro [] :b)
                (defn do-macro [] (my-macro))
                (do-macro)))))))
+
+(deftest nested-dynamic-declaration
+  (testing "vars :dynamic meta data is applied immediately to vars declared anywhere"
+    (is (= 10
+          (eval
+            '(do
+               (list
+                 (declare ^:dynamic p)
+                 (defn q [] @p))
+               (binding [p (atom 10)]
+                 (q))))))))
\ No newline at end of file
