@@ -27,7 +27,7 @@
         avec (into [] arange)
         alist (into () arange)
         obj-array (into-array arange)
-        int-array (into-array Integer/TYPE arange)
+        int-array (into-array Integer/TYPE (map #(Integer. (int %)) arange))
         long-array (into-array Long/TYPE arange)
         float-array (into-array Float/TYPE arange)
         char-array (into-array Character/TYPE (map char arange))
@@ -40,7 +40,7 @@
         double-vec (into (vector-of :double) arange)
         byte-vec (into (vector-of :byte) (map byte arange))
         all-true (into-array Boolean/TYPE (repeat 10 true))]
-    (is (= 4950
+    (is (== 4950
            (reduce + arange)
            (reduce + avec)
            (reduce + alist)
@@ -57,7 +57,7 @@
            (reduce int+ char-vec)
            (reduce + double-vec)
            (reduce int+ byte-vec)))
-    (is (= 4951
+    (is (== 4951
            (reduce + 1 arange)
            (reduce + 1 avec)
            (reduce + 1 alist)
@@ -243,11 +243,11 @@
 
 
 (deftest test-first
-  (is (thrown? IllegalArgumentException (first)))
+  ;(is (thrown? Exception (first)))
   (is (thrown? IllegalArgumentException (first true)))
   (is (thrown? IllegalArgumentException (first false)))
   (is (thrown? IllegalArgumentException (first 1)))
-  (is (thrown? IllegalArgumentException (first 1 2)))
+  ;(is (thrown? IllegalArgumentException (first 1 2)))
   (is (thrown? IllegalArgumentException (first \a)))
   (is (thrown? IllegalArgumentException (first 's)))
   (is (thrown? IllegalArgumentException (first :k)))
@@ -310,11 +310,11 @@
 
 
 (deftest test-next
-  (is (thrown? IllegalArgumentException (next)))
+ ; (is (thrown? IllegalArgumentException (next)))
   (is (thrown? IllegalArgumentException (next true)))
   (is (thrown? IllegalArgumentException (next false)))
   (is (thrown? IllegalArgumentException (next 1)))
-  (is (thrown? IllegalArgumentException (next 1 2)))
+  ;(is (thrown? IllegalArgumentException (next 1 2)))
   (is (thrown? IllegalArgumentException (next \a)))
   (is (thrown? IllegalArgumentException (next 's)))
   (is (thrown? IllegalArgumentException (next :k)))
@@ -445,7 +445,7 @@
 ;; (ffirst coll) = (first (first coll))
 ;;
 (deftest test-ffirst
-  (is (thrown? IllegalArgumentException (ffirst)))
+;  (is (thrown? IllegalArgumentException (ffirst)))
   (are [x y] (= x y)
     (ffirst nil) nil
 
@@ -465,7 +465,7 @@
 ;; (fnext coll) = (first (next coll)) = (second coll)
 ;;
 (deftest test-fnext
-  (is (thrown? IllegalArgumentException (fnext)))
+;  (is (thrown? IllegalArgumentException (fnext)))
   (are [x y] (= x y)
     (fnext nil) nil
 
@@ -489,7 +489,7 @@
 ;; (nfirst coll) = (next (first coll))
 ;;
 (deftest test-nfirst
-  (is (thrown? IllegalArgumentException (nfirst)))
+;  (is (thrown? IllegalArgumentException (nfirst)))
   (are [x y] (= x y)
     (nfirst nil) nil
 
@@ -509,7 +509,7 @@
 ;; (nnext coll) = (next (next coll))
 ;;
 (deftest test-nnext
-  (is (thrown? IllegalArgumentException (nnext)))
+;  (is (thrown? IllegalArgumentException (nnext)))
   (are [x y] (= x y)
     (nnext nil) nil
 
@@ -877,7 +877,7 @@
 
 
 (deftest test-repeat
-  (is (thrown? IllegalArgumentException (repeat)))
+  ;(is (thrown? IllegalArgumentException (repeat)))
 
   ; infinite sequence => use take
   (are [x y] (= x y)
